package filters

import (
	"strings"

	"github.com/aquasecurity/tracee/pkg/events"
)

type RetFilter struct {
	filters map[events.ID]*IntFilter
	enabled bool
}

func NewRetFilter() *RetFilter {
	return &RetFilter{
		filters: map[events.ID]*IntFilter{},
		enabled: false,
	}
}

func (filter *RetFilter) Filter(eventID events.ID, retVal int64) bool {
	if !filter.Enabled() {
		return true
	}
	if filter, ok := filter.filters[eventID]; ok {
		return filter.Filter(retVal)
	}
	return true
}

func (filter *RetFilter) Enable() {
	filter.enabled = true
	for _, f := range filter.filters {
		f.Enable()
	}
}

func (filter *RetFilter) Disable() {
	filter.enabled = false
	for _, f := range filter.filters {
		f.Disable()
	}
}

func (filter *RetFilter) Enabled() bool {
	return filter.enabled
}

func (filter *RetFilter) Parse(filterName string, operatorAndValues string, eventsNameToID map[string]events.ID) error {
	// Ret filter has the following format: "event.retval=val"
	// filterName have the format event.retval, and operatorAndValues have the format "=val"
	splitFilter := strings.Split(filterName, ".")
	if len(splitFilter) != 2 || splitFilter[1] != "retval" {
		return InvalidExpression(filterName + operatorAndValues)
	}

	eventName := splitFilter[0]
	if eventName == "" {
		return InvalidExpression(filterName + operatorAndValues)
	}

	id, ok := eventsNameToID[eventName]
	if !ok {
		return InvalidEventName(eventName)
	}

	if _, ok := filter.filters[id]; !ok {
		filter.filters[id] = NewIntFilter()
	}

	intFilter := filter.filters[id]

	// Treat operatorAndValues as an int filter to avoid code duplication
	err := intFilter.Parse(operatorAndValues)
	if err != nil {
		return err
	}

	filter.filters[id] = intFilter

	filter.Enable()

	return nil
}
